'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _defineProperty = require('babel-runtime/core-js/object/define-property');

var _defineProperty2 = _interopRequireDefault(_defineProperty);

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _isNode = require('../../helpers/isNode');

var _isNode2 = _interopRequireDefault(_isNode);

var _hash = require('@s-ui/js/lib/hash');

var _stringOrIntToMs = require('../../helpers/stringOrIntToMs');

var _stringOrIntToMs2 = _interopRequireDefault(_stringOrIntToMs);

var _LRU = require('./algorithms/LRU');

var _LRU2 = _interopRequireDefault(_LRU);

var _LFU = require('./algorithms/LFU');

var _LFU2 = _interopRequireDefault(_LFU);

var _tracker = require('./tracker');

var _tracker2 = _interopRequireDefault(_tracker);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var DEFAULT_TTL = 500;

var isPromise = function isPromise(obj) {
  return typeof obj !== 'undefined' && typeof obj.then === 'function';
};

var caches = {};

var _cache = function _cache() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      ttl = _ref.ttl,
      target = _ref.target,
      fnName = _ref.fnName,
      instance = _ref.instance,
      original = _ref.original,
      server = _ref.server,
      algorithm = _ref.algorithm,
      host = _ref.host,
      port = _ref.port,
      segmentation = _ref.segmentation,
      size = _ref.size;

  caches[fnName] = caches[fnName] || (algorithm === 'lru' ? new _LRU2.default({ size: size }) : algorithm === 'lfu' ? new _LFU2.default({ size: size }) : new Error('[cv-decorators::cache] unknow algorithm: ' + algorithm));

  var cache = caches[fnName];
  var tracker = new _tracker2.default({ algorithm: algorithm, host: host, port: port, fnName: fnName, segmentation: segmentation });

  return function () {
    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    var key = target.constructor.name + '::' + fnName + '::' + (0, _hash.createHash)((0, _stringify2.default)(args));
    var now = Date.now();
    if (cache.get(key) === undefined) {
      tracker._updateStats({ action: _tracker2.default.ACTION_MISSING });
      cache.set(key, { createdAt: now, returns: original.apply(instance, args) });
    } else {
      tracker._updateStats({ action: _tracker2.default.ACTION_HIT });
    }

    tracker.track();

    if (isPromise(cache.get(key).returns)) {
      cache.get(key).returns.catch(function () {
        return cache.del(key);
      });
    }

    if (now - cache.get(key).createdAt > ttl) {
      cache.del(key);
    }

    return cache.get(key) !== undefined ? cache.get(key).returns : original.apply(instance, args);
  };
};

exports.default = function () {
  var _ref2 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      _ref2$ttl = _ref2.ttl,
      ttl = _ref2$ttl === undefined ? DEFAULT_TTL : _ref2$ttl,
      _ref2$server = _ref2.server,
      server = _ref2$server === undefined ? false : _ref2$server,
      _ref2$algorithm = _ref2.algorithm,
      algorithm = _ref2$algorithm === undefined ? 'lru' : _ref2$algorithm,
      host = _ref2.trackTo,
      port = _ref2.port,
      segmentation = _ref2.segmentation,
      size = _ref2.size;

  var timeToLife = (0, _stringOrIntToMs2.default)({ ttl: ttl }) || DEFAULT_TTL;
  return function (target, fnName, descriptor) {
    var fn = descriptor.value,
        configurable = descriptor.configurable,
        enumerable = descriptor.enumerable;


    if (_isNode2.default && !server) {
      return descriptor;
    }

    // https://github.com/jayphelps/core-decorators.js/blob/master/src/autobind.js
    return (0, _assign2.default)({}, {
      configurable: configurable,
      enumerable: enumerable,
      get: function get() {
        if (this === target) {
          return fn;
        }
        var _fnCached = _cache({
          ttl: timeToLife, target: target, fnName: fnName, instance: this, original: fn, server: server, algorithm: algorithm, host: host, port: port, segmentation: segmentation, size: size
        });

        (0, _defineProperty2.default)(this, fnName, {
          configurable: true,
          writable: true,
          enumerable: false,
          value: _fnCached
        });
        return _fnCached;
      },
      set: function set(newValue) {
        (0, _defineProperty2.default)(this, fnName, {
          configurable: true,
          writable: true,
          enumerable: true,
          value: newValue
        });

        return newValue;
      }
    });
  };
};