'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var isPromise = function isPromise(obj) {
  return typeof obj !== 'undefined' && typeof obj.then === 'function';
};

var defaultErrorHandler = function defaultErrorHandler(err) {
  console.error(err);
};

var dispatchToListeners = function dispatchToListeners(_ref) {
  var onError = _ref.onError,
      onNext = _ref.onNext,
      params = _ref.params,
      result = _ref.result;

  if (isPromise(result)) {
    result.then(function (value) {
      return onNext({ params: params, result: value });
    }).catch(function (error) {
      return onError({ params: params, error: error });
    });
  } else if (result) {
    onNext({ params: params, result: result });
  }
};

var createSubscription = function createSubscription(proto, method, originalMethod) {
  var onNextListeners = [];
  var onErrorListeners = [];

  proto[method] = function () {
    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    var params = args;
    try {
      var result = originalMethod.apply(this, args);
      dispatchToListeners({ onError: onError, onNext: onNext, params: params, result: result });
      return result;
    } catch (error) {
      onError({ params: params, error: error });
      throw error;
    }
  };

  var onNext = function onNext(_ref2) {
    var params = _ref2.params,
        result = _ref2.result;

    onNextListeners.forEach(function (listener) {
      return listener({ params: params, result: result });
    });
  };

  var onError = function onError(_ref3) {
    var params = _ref3.params,
        error = _ref3.error;

    onErrorListeners.forEach(function (listener) {
      return listener({ params: params, error: error });
    });
  };

  var subscribe = function subscribe() {
    var onNext = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : function () {};
    var onError = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : defaultErrorHandler;

    onNextListeners.push(onNext);
    onErrorListeners.push(onError);

    return {
      dispose: function dispose() {
        onNextListeners = onNextListeners.filter(function (l) {
          return l !== onNext;
        });
        onErrorListeners = onErrorListeners.filter(function (l) {
          return l !== onError;
        });
      }
    };
  };

  return { subscribe: subscribe };
};

var reducer = function reducer(Target, proto, method) {
  var originalMethod = Target.prototype[method];
  proto.$ = proto.$ || {};
  proto.$[method] = createSubscription(proto, method, originalMethod);
  return proto;
};

exports.default = function () {
  for (var _len2 = arguments.length, methods = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
    methods[_key2] = arguments[_key2];
  }

  return function (Target) {
    (0, _assign2.default)(Target.prototype, methods.filter(function (method) {
      return !!Target.prototype[method];
    }).reduce(reducer.bind(null, Target), {}));
    return Target;
  };
};